
import unittest
from unittest import skip

from pycozmo.image_encoder import ImageDecoder, image_to_str
from pycozmo.util import hex_load


class TestImageDecoder(unittest.TestCase):

    @staticmethod
    def _decode(seq):
        buffer = hex_load(seq)
        decoder = ImageDecoder(buffer)
        decoder.decode()
        return decoder

    def _compare(self, expected, decoder):
        actual = image_to_str(decoder.image)
        self.assertEqual(expected.strip(), actual.strip())

    def test_blank(self):
        seq = "3f:3f"
        expected = """
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)
        self.assertEqual(128, decoder.x)
        self.assertEqual(0, decoder.y)

    def test_fill_screen(self):
        seq = "fd:7f:7e"
        # FIXME: Is this the expected value?
        expected = """
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)
        self.assertEqual(128, decoder.x)
        self.assertEqual(0, decoder.y)

    def test_top_left(self):
        seq = "81:3f:3e"
        expected = """
1...............................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_top_left_5(self):
        seq = "03:81:3f:3a"
        expected = """
....1...........................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_top_left_1_8(self):
        seq = "81:06:81:3f:36"
        expected = """
1.......1.......................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_top_left_line(self):
        seq = "81:46:3f:37"
        expected = """
11111111........................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_top_line(self):
        seq = "82:7f:7e"
        expected = """
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_bottom_line(self):
        seq = "f8:82:7f:7e"
        expected = """
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_left_line(self):
        seq = "fe:3f:3e"
        expected = """
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
1...............................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_right_line(self):
        seq = "3f:3e:fe"
        expected = """
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
...............................................................................................................................1
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_columns(self):
        seq = "fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00:fd:00"
        expected = """
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_rect(self):
        seq = "fe:82:f4:82:7f:7c:fe"
        expected = """
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_rect2(self):
        seq = "00:fe:82:f4:82:7f:7a:fe:00"
        expected = """
.111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.1............................................................................................................................1.
.111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111.
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_rect3(self):
        seq = "01:80:f6:80:80:82:ec:82:7f:78:80:f6:01"
        expected = """
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_rect4(self):
        seq = "fe:82:f4:82:fe:86:ec:86:7f:78:fe:82:f4:82:fe"
        expected = """
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
1..............................................................................................................................1
11111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111111
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_diagonal(self):
        seq = "82:42:80:82:42:84:82:42:88:82:42:8c:82:42:90:82:42:94:82:42:98:82:42:9c:82:42:a0:82:42:a4:82:42:a8:82:42:ac:82:42:b0:82:42:b4:82:42:b8:82:42:bc:82:42:c0:82:42:c4:82:42:c8:82:42:cc:82:42:d0:82:42:d4:82:42:d8:82:42:dc:82:42:e0:82:42:e4:82:42:e8:82:42:ec:82:42:f0:82:42:f4:82:42:f8:82:42"
        expected = """
1111............................................................................................................................
....1111........................................................................................................................
........1111....................................................................................................................
............1111................................................................................................................
................1111............................................................................................................
....................1111........................................................................................................
........................1111....................................................................................................
............................1111................................................................................................
................................1111............................................................................................
....................................1111........................................................................................
........................................1111....................................................................................
............................................1111................................................................................
................................................1111............................................................................
....................................................1111........................................................................
........................................................1111....................................................................
............................................................1111................................................................
................................................................1111............................................................
....................................................................1111........................................................
........................................................................1111....................................................
............................................................................1111................................................
................................................................................1111............................................
....................................................................................1111........................................
........................................................................................1111....................................
............................................................................................1111................................
................................................................................................1111............................
....................................................................................................1111........................
........................................................................................................1111....................
............................................................................................................1111................
................................................................................................................1111............
....................................................................................................................1111........
........................................................................................................................1111....
............................................................................................................................1111
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_diagonal2(self):
        seq = "f8:82:42:f4:82:42:f0:82:42:ec:82:42:e8:82:42:e4:82:42:e0:82:42:dc:82:42:d8:82:42:d4:82:42:d0:82:42:cc:82:42:c8:82:42:c4:82:42:c0:82:42:bc:82:42:b8:82:42:b4:82:42:b0:82:42:ac:82:42:a8:82:42:a4:82:42:a0:82:42:9c:82:42:98:82:42:94:82:42:90:82:42:8c:82:42:88:82:42:84:82:42:80:82:42:82:42"
        expected = """
............................................................................................................................1111
........................................................................................................................1111....
....................................................................................................................1111........
................................................................................................................1111............
............................................................................................................1111................
........................................................................................................1111....................
....................................................................................................1111........................
................................................................................................1111............................
............................................................................................1111................................
........................................................................................1111....................................
....................................................................................1111........................................
................................................................................1111............................................
............................................................................1111................................................
........................................................................1111....................................................
....................................................................1111........................................................
................................................................1111............................................................
............................................................1111................................................................
........................................................1111....................................................................
....................................................1111........................................................................
................................................1111............................................................................
............................................1111................................................................................
........................................1111....................................................................................
....................................1111........................................................................................
................................1111............................................................................................
............................1111................................................................................................
........................1111....................................................................................................
....................1111........................................................................................................
................1111............................................................................................................
............1111................................................................................................................
........1111....................................................................................................................
....1111........................................................................................................................
1111............................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_eyes(self):
        seq = "11:9c:b6:a4:98:be:40:94:c6:40:90:ce:5b:94:c6:9c:98:be:a0:9c:b6:06:a0:ae:40:9c:b6:40:98:be:5d:9c:b6:40:a0:ae:1b"
        expected = """
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_eyes2(self):
        seq = "17:f4:81:41:f4:85:f4:81:83:5b:f4:81:40:09:f4:81:40:f4:85:f4:81:83:4c:f8:83:4d:f8:81:19"
        expected = """
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)

    def test_eyes3(self):
        seq = "16:a0:b6:41:9c:be:40:98:c6:5b:9c:be:9c:a0:b6:40:06:a4:ae:a4:a0:b6:40:9c:be:40:98:c6:5b:9c:be:40:a0:b6:40:16"
        expected = """
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
................................................................................................................................
"""
        decoder = self._decode(seq)
        self._compare(expected, decoder)
